------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com   
------------------------------------------

local LrApplication = import 'LrApplication'
local LrBinding = import 'LrBinding'
local LrView = import 'LrView'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin() 

local myLogger = LrLogger('PhotomatixLogger')
myLogger:enable("print") 

local cntPhotos=0
local folderPath=''

CreateHdrSettings = {}

function CreateHdrSettings.outputToLog(message)
	myLogger:trace(message)
end


function CreateHdrSettings.debutCommun(fic1, fic2, minSeuilLen)
	local resStr=''
	local i=1
	local lenFicMin=#fic1
	if #fic2<lenFicMin then 
		lenFicMin=#fic2
	end
	while (i<=lenFicMin) do
		if string.sub(fic1, i, i) ~= string.sub(fic2, i, i) then
			break
		end
		i=i+1
	end	
	if (i>minSeuilLen) then
		resStr=string.sub(fic1, 1, i-1)
	end
	return resStr
end



function CreateHdrSettings.defaultFileName( )
	local combinedFileName=''
	
	if prefs.LrVersion<2 then
		return combinedFileName
	end
	local activeCatalog = LrApplication.activeCatalog()
	activeCatalog:withReadAccessDo( function()
		local photos = activeCatalog.targetPhotos	

		cntPhotos=#photos  --		CreateHdrSettings.outputToLog(string.format('cntPhotos: %d',cntPhotos))
		folderPath=LrPathUtils.parent(photos[1].path)  --		CreateHdrSettings.outputToLog(string.format('folderPath: %s',folderPath))

		local iPhoto=1
		local commonStr=''
		local firstFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[1].path))  ---		CreateHdrSettings.outputToLog(string.format('firstFileName%d:  %s',iPhoto,firstFileName))
		local maxLen=0
		for iPhoto=2, cntPhotos, 1 do		
			local curFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[iPhoto].path))   --			CreateHdrSettings.outputToLog(string.format('curPath%d:  %s',iPhoto,photos[iPhoto].path))
			if (iPhoto==2) then
				commonStr=CreateHdrSettings.debutCommun(firstFileName, curFileName, 2) 
			else 
				if #commonStr>1 then
					commonStr=CreateHdrSettings.debutCommun(commonStr, curFileName, 2) 
				end
			end	
			if #curFileName>maxLen then 
				maxLen=#curFileName
			end
		end

		if #commonStr>1 then
			if ((maxLen - #commonStr) <= 8) then
				combinedFileName=firstFileName
				for iPhoto=2, cntPhotos, 1 do		
					local curFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[iPhoto].path))
					combinedFileName=string.format('%s_%s', combinedFileName, string.sub(curFileName, #commonStr+1))  --					CreateHdrSettings.outputToLog(string.format('combinedFileName%d:  %s',iPhoto,combinedFileName))
				end
			end
		end	
		if ((#combinedFileName<2) or (cntPhotos > 7)) then
			combinedFileName=string.format('%sAnd%dmore', firstFileName, cntPhotos-1)
		end
		
	end )

	return combinedFileName
end


function CreateHdrSettings.getOptions( )

	local isLr2=false
	local titleBox2="Handling processed image (requires Lightroom 2)"

	if prefs.LrVersion==nil then
		LrDialogs.message("Plugin did not start correctly","Please check whether you invoked the plugin by choosing 'File > Plug-in Extras'","info")
		return
	end

	if prefs.LrVersion>1 then
		isLr2=true
		titleBox2=LOC "$$$/Photomatix/Settings/BoxTitle=Saving"
	end

	LrFunctionContext.callWithContext( 'CreateHdrSettings', function( context )

	local f = LrView.osFactory() 
	local props = LrBinding.makePropertyTable( context ) 
	props.processType = 'CreateHDR'
	props.chkAlign=true
	props.alignType='1'
	props.chkCrop=true
	props.chkPerspective=true
	props.chkReduceNoise=false
	props.nrWhat='0'
	props.chkGhosting=false
	props.ghostingType='1'
	props.ghostingDetection='normal'
	props.outputFormat='tif32'
	props.stackWith=false
	props.waitForPhotomatix=true
	props.outputName=CreateHdrSettings.defaultFileName()  --	CreateHdrSettings.outputToLog(string.format('props.outputName:  %s',props.outputName))

	if ((prefs.processType~=nil) and (prefs.processType~='none')) then
		props.processType=prefs.processType
	end
	if (prefs.chkAlign~=nil) then
		props.chkAlign=prefs.chkAlign
	end
	if (prefs.chkCrop~=nil) then
		props.chkCrop=prefs.chkCrop
	end
	if (prefs.alignType~=nil) then
		if ((prefs.alignType=='1')or(prefs.alignType=='2')) then
			props.alignType=prefs.alignType
		end
	end
	if (prefs.chkPerspective~=nil) then
		props.chkPerspective=prefs.chkCrop
	end
	if (prefs.ghostingType~=nil) then
	        if ((prefs.ghostingType=='1') or (prefs.ghostingType=='2')) then
		        props.ghostingType=prefs.ghostingType
		end
	end
	if (prefs.chkPerspective~=nil) then
		props.chkPerspective=prefs.chkPerspective
	end
	if ((prefs.chkGhosting)and((prefs.ghostingType=='1')or(prefs.ghostingType=='2'))) then
		props.chkGhosting=prefs.chkGhosting
	end
	if (prefs.ghostingDetection~=nil) then
		props.ghostingDetection=prefs.ghostingDetection
	end
	if (prefs.chkReduceNoise) then
		props.chkReduceNoise=prefs.chkReduceNoise
	end
	if (prefs.nrWhat~=nil) then
		props.nrWhat=prefs.nrWhat
	end
	if (prefs.outputFormat~=nil) then
		props.outputFormat=prefs.outputFormat
	end
	if (prefs.waitForPhotomatix) then
		props.waitForPhotomatix=prefs.waitForPhotomatix
	end
	if (prefs.stackWith) then
		props.stackWith=prefs.stackWith
	end
	

	local multipleFilesSettings = f:column {
		bind_to_object = props,
		spacing = f:dialog_spacing(),
		
		f:row {
			f:static_text {
				title = LOC ("$$$/Photomatix/Settings/Heading=You have selected ^1 photos to be merged to HDR by Photomatix Pro ^nand saved as a 32-bit TIFF file.^nPlease set the options for merging the photos and saving.",cntPhotos),
				warps=true,
				alignment = 'left',
			},
		},

		f:group_box {
			title = LOC "$$$/Photomatix/Settings/CreateHDR/Title=Pre-processing and merging",
			show_title=true,
			fill_horizontal=1,
			font = "<system>",

		f:row {	
			spacing = f:control_spacing() / 2,
			f:column {
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/Align/Checkbox=Align images                      ",
					tooltip=LOC "$$$/Photomatix/Settings/Align/Tooltip=Check this option if your source images are hand-held photographs.",
					value = LrView.bind ("chkAlign"),
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:column {
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/Crop/Checkbox=Crop aligned result",
					tooltip=LOC "$$$/Photomatix/Settings/Crop/Tooltip=Checking this option will remove the unneeded borders resulting from the alignment adjustments.",
					value = LrView.bind ("chkCrop"),
					enabled = LrView.bind("chkAlign"),
					checked_value = true,
					unchecked_value = false,
				},
			},
		},

		f:row {	
			f:column {
				spacing = f:control_spacing() *2/3,
				margin_left = 15,
				f:radio_button {
					title =LOC "$$$/Photomatix/Settings/AlignMethod1=by correcting horizontal and vertical shifts",
					tooltip=LOC "$$$/Photomatix/Settings/AlignMethod1/Tooltip=This alignment method is fast but corrects for translation only.",
					value = LrView.bind ("alignType"),
					checked_value = '1',
					enabled = LrView.bind("chkAlign"),
				},
				f:row {	
				f:column {
					f:radio_button {
						title =LOC "$$$/Photomatix/Settings/AlignMethod2=by matching features",
						tooltip=LOC "$$$/Photomatix/Settings/AlignMethod2/Tooltip=This alignment method corrects for translation, rotation and scaling (Note: rotation correction requires resampling).",
						value = LrView.bind ("alignType"),
						checked_value = '2',
						enabled = LrView.bind("chkAlign"),
					},
				},
				f:column {
					f:checkbox {
						title = LOC "$$$/Photomatix/Settings/PerspectiveCorrection/Checkbox=include perspective correction",
						tooltip=LOC "$$$/Photomatix/Settings/PerspectiveCorrection/Tooltip=When checked, the alignment tries to correct for perspective distortion (in addition to correcting for translation, rotation and scaling differences).",
						value = LrView.bind ("chkPerspective"),
						checked_value = true,
						unchecked_value = false,
						enabled = LrView.bind {
							keys = { 'chkAlign', 'alignType' },
							operation = function( binder, values, fromTable )
							   if fromTable then
							      return values.chkAlign and values.alignType == '2'
							   end
							   return LrBinding.kUnsupportedDirection
							end }
					},
				},
				},
			},				
		},


		f:row {	
			spacing = f:control_spacing() / 3,

			f:checkbox {
				title = LOC "$$$/Photomatix/Settings/Deghost=Remove ghosts",
				tooltip=LOC "$$$/Photomatix/Settings/Deghost/Tooltip=Only check this option if your scene includes elements that moved between the bracketed shots. Otherwise, leave it unchecked, as removing ghosts lowers the quality of the merged image.",
				value = LrView.bind ("chkGhosting"),
				checked_value = true,
				unchecked_value = false,
			},
		},

		f:row {	
			spacing = f:control_spacing() / 2,
			f:column {			
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/NR=Reduce noise",
					tooltip=LOC "$$$/Photomatix/Settings/NR/Tooltip=High quality noise reduction but significantly increases processing times. It is better to disable Lightroom Noise Reduction when you check this option",
					value = LrView.bind ("chkReduceNoise"),
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:column {			
				f:popup_menu {
					value = LrView.bind ("nrWhat"),
					items = {
						{ title = LOC "$$$/Photomatix/Settings/NR/UnderOnly=Underexposed image(s) only", value = "0" },
						{ title = LOC "$$$/Photomatix/Settings/NR/NormalUnder=Normal exposure & underexposed", value = "1" },
						{ title = LOC "$$$/Photomatix/Settings/NR/All=All source images", value = "2" },
					},
					fill_horizontal=0.4,
					enabled = LrView.bind("chkReduceNoise"),
				},
			},
		},


		f:row {					
			spacing = f:control_spacing(),
		},
		
		},
		
		f:group_box {
			title = titleBox2,
			fill_horizontal=1,
			show_title=true,
			font = "<system>",

		f:row {	
			f:checkbox {
				title = LOC "$$$/Photomatix/Settings/Stack/Multiple=Stack with first selected photo",
				value = LrView.bind ("stackWith"), 
				alignment = 'left',
				place_horizontal=0.15,
			},			
		},
		f:row {	
			spacing = f:control_spacing(),
			f:static_text {
				title = LOC "$$$/Photomatix/Settings/FileName=File name:",
				alignment = 'right',
			},			
			f:edit_field { 
				value = LrView.bind ("outputName"), 
				fill_horizontal=1,
			},
		}, 

--[[	f:row {	
			f:static_text {
				title = LOC "$$$/Photomatix/Settings/OutputFormat=Output Format:",
				alignment = 'right',
				place_horizontal=0.5
			},			
			f:popup_menu {
				tooltip=LOC "$$$/Photomatix/Settings/OutputFormat/Tooltip=Format for saving the resulting tone mapped or fused image.",
				value = LrView.bind ("outputFormat"),
				items = {
				{ title = "JPEG", value = "jpg" },
				{ title = "TIFF 8-bit", value = "tif8" },
				{ title = "TIFF 16-bit", value = "tif16" },
				},
				fill_horizontal=0.4,
			},
		},
--]]		

		},
	}

	if cntPhotos==1 then
		LrDialogs.message(LOC "$$$/Photomatix/General/Note=Note", 
			LOC "$$$/Photomatix/General/SinglePhotoMerge=You selected a single photo.  To create a 32-bit HDR file, please select bracketed photos, i.e. photos of the same scene taken under different exposures.", "info")
	else

		local result = LrDialogs.presentModalDialog( {
			title = LOC "$$$/Photomatix/CreateHdrSettings/Title=Create 32-bit HDR Options",
			contents = multipleFilesSettings,
			actionVerb = LOC "$$$/Photomatix/General/CreateHDR=Create HDR",
		} )

		if (result=='ok') then
			prefs.processType='CreateHDR'
			prefs.chkAlign=props.chkAlign
			prefs.chkCrop=props.chkCrop
			prefs.alignType=props.alignType
			prefs.chkPerspective=props.chkPerspective
			prefs.chkGhosting=props.chkGhosting
			prefs.ghostingType=props.ghostingType
			prefs.ghostingDetection=props.ghostingDetection
			prefs.chkReduceNoise=props.chkReduceNoise
			prefs.nrWhat=props.nrWhat
			prefs.waitForPhotomatix=props.waitForPhotomatix 
			prefs.outputFormat=props.outputFormat
			prefs.stackWith=props.stackWith
			local outputExt=string.sub(props.outputFormat, 1, 3)
			prefs.outputPath=LrPathUtils.child(folderPath, LrPathUtils.addExtension(props.outputName, outputExt))
		else
			prefs.processType='none'
		end	
	
	end
end )

end
